package cd.casic.module.process.core.resolver;

import cd.casic.module.process.core.context.ApplicationContextContext;
import org.springframework.beans.BeansException;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.core.env.StandardEnvironment;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.type.AnnotationMetadata;
import org.springframework.core.type.ClassMetadata;
import org.springframework.core.type.classreading.MetadataReader;
import org.springframework.core.type.classreading.SimpleMetadataReaderFactory;
import org.springframework.util.ClassUtils;
import org.springframework.util.StringUtils;

import java.lang.annotation.Annotation;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

public class AnnotationResourceResolver {
    private static final PathMatchingResourcePatternResolver resolver = new PathMatchingResourcePatternResolver();
    private static final SimpleMetadataReaderFactory register = new SimpleMetadataReaderFactory();
    private static final StandardEnvironment environment = new StandardEnvironment();

    public AnnotationResourceResolver() {
    }

    public static Set<Class> resolveBeans(String pkgPath, Class<? extends Annotation> annoClazz) {
        Set<Class> clazzSet = new HashSet();
        ConfigurableApplicationContext applicationContext = ApplicationContextContext.getApplicationContext();
        Map<String, Object> beansWithAnnotationMap = applicationContext.getBeansWithAnnotation(annoClazz);
        Iterator var5 = beansWithAnnotationMap.entrySet().iterator();

        while(var5.hasNext()) {
            Map.Entry<String, Object> entry = (Map.Entry)var5.next();
            Object object = entry.getValue();
            Class clazz = object.getClass();
            clazzSet.add(clazz);
        }

        return clazzSet;
    }

    public static Set<Class> resolve(String pkgPath, Class<? extends Annotation> annoClazz) {
        Set<Class> paths = new HashSet();

        try {
            String pathPackage = getResourcePath(pkgPath);
            Resource[] resources = resolver.getResources(pathPackage);

            for(int i = 0; i < resources.length; ++i) {
                Resource resource = resources[i];
                MetadataReader metadataReader = register.getMetadataReader(resource);
                AnnotationMetadata annotationMetadata = metadataReader.getAnnotationMetadata();
                if (annotationMetadata.hasAnnotation(annoClazz.getName())) {
                    ClassMetadata classMetadata = metadataReader.getClassMetadata();
                    String className = classMetadata.getClassName();
                    ClassLoader classLoader = getClassLoader();
                    Class<?> clazz = classLoader.loadClass(className);
                    paths.add(clazz);
                }
            }

            return paths;
        } catch (Exception var13) {
            throw new RuntimeException(var13);
        }
    }

    public static Set<Class> resolveFromBeans(String pkgPath, Class<? extends Annotation> annoClazz) {
        Set<Class> paths = new HashSet();
        ConfigurableApplicationContext applicationContext = ApplicationContextContext.getApplicationContext();

        try {
            String pathPackage = getResourcePath(pkgPath);
            Resource[] resources = resolver.getResources(pathPackage);

            for(int i = 0; i < resources.length; ++i) {
                Resource resource = resources[i];
                MetadataReader metadataReader = register.getMetadataReader(resource);
                AnnotationMetadata annotationMetadata = metadataReader.getAnnotationMetadata();
                if (annotationMetadata.hasAnnotation(annoClazz.getName())) {
                    ClassMetadata classMetadata = metadataReader.getClassMetadata();
                    String className = classMetadata.getClassName();
                    ClassLoader classLoader = getClassLoader();
                    Class<?> clazz = classLoader.loadClass(className);

                    try {
                        Object bean = applicationContext.getBean(clazz);
                        if (bean == null) {
                            continue;
                        }
                    } catch (BeansException var15) {
                        continue;
                    }

                    paths.add(clazz);
                }
            }

            return paths;
        } catch (Exception var16) {
            throw new RuntimeException(var16);
        }
    }

    public static String getResourcePath(String packagePath) {
        if (StringUtils.isEmpty(packagePath)) {
            return "";
        } else {
            String resourcePath = "classpath*:" + ClassUtils.convertClassNameToResourcePath(environment.resolveRequiredPlaceholders(packagePath)) + "/**/*.class";
            return resourcePath;
        }
    }

    static ClassLoader getClassLoader() {
        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        if (classLoader == null) {
            classLoader = ClassLoader.getSystemClassLoader();
        }

        return classLoader;
    }
}
