package cd.casic.module.process.task.artifact.service;

/**
 * @ClassName TaskArtifactService
 * @Author hopeli
 * @Date 2025/5/2 19:02
 * @Version 1.0
 */

import cd.casic.ci.commons.annotation.join.FindAll;
import cd.casic.ci.commons.annotation.join.FindList;
import cd.casic.ci.commons.annotation.join.FindOne;
import cd.casic.ci.commons.annotation.join.JoinProvider;
import cd.casic.module.process.task.artifact.model.TaskArtifact;

import java.util.List;

/**
 * 任务推送制品服务接口
 */
@JoinProvider(model = TaskArtifact.class)
public interface TaskArtifactService {

    /**
     * 创建流水线推送制品
     * @param taskArtifact 流水线推送制品
     * @return 流水线推送制品id
     */
    String createProduct(TaskArtifact taskArtifact);

    /**
     * 删除流水线推送制品
     * @param productId 流水线推送制品id
     */
    void deleteProduct(String productId);


    /**
     * 根据配置id查询任务
     * @param taskId 配置id
     * @return 任务
     */
    TaskArtifact findOneArtifactByAuth(String taskId);


    /**
     * 更新推送制品信息
     * @param taskArtifact 信息
     */
    void updateProduct(TaskArtifact taskArtifact);

    /**
     * 查询推送制品信息
     * @param artifactId id
     * @return 信息
     */
    @FindOne
    TaskArtifact findOneProduct(String artifactId);

    /**
     * 查询所有流水线推送制品
     * @return 流水线推送制品列表
     */
    @FindAll
    List<TaskArtifact> findAllProduct();


    /**
     * 根据ID列表查询流水线推送制品
     * @param idList ID列表
     * @return 流水线推送制品列表
     */
    @FindList
    List<TaskArtifact> findAllProductList(List<String> idList);

}
