package cd.casic.module.process.task.artifact.service.impl;

import cd.casic.ci.commons.bean.process.setting.AuthHost;
import cd.casic.ci.commons.bean.process.setting.AuthThird;
import cd.casic.ci.commons.annotation.rpc.Exporter;
import cd.casic.module.process.process.setting.AuthHostService;
import cd.casic.module.process.process.setting.AuthThirdService;
import cd.casic.module.process.support.util.util.PipelineFinal;
import cd.casic.module.process.task.artifact.dao.TaskArtifactDao;
import cd.casic.module.process.task.artifact.dataobject.TaskArtifactDo;
import cd.casic.module.process.task.artifact.model.TaskArtifact;
import cd.casic.module.process.task.artifact.service.TaskArtifactService;
import cd.casic.module.process.toolkit.beans.BeanMapper;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 * @ClassName TaskArtifactServiceImpl
 * @Author hopeli
 * @Date 2025/5/2 19:34
 * @Version 1.0
 */


@Service
@Exporter
public class TaskArtifactServiceImpl implements TaskArtifactService {

    @Resource
    private TaskArtifactDao productDao;

    @Resource
    private AuthThirdService thirdServer;

    @Resource
    private AuthHostService hostServer;

    /**
     * 创建流水线推送制品
     * @param taskArtifact 流水线推送制品
     * @return 流水线推送制品id
     */
    @Override
    public String createProduct(TaskArtifact taskArtifact) {
        TaskArtifactDo taskArtifactEntity = BeanMapper.map(taskArtifact, TaskArtifactDo.class);
        return productDao.createProduct(taskArtifactEntity);
    }

    /**
     * 根据配置id查询任务
     * @return 任务
     */
    @Override
    public TaskArtifact findOneArtifactByAuth(String taskId){

        TaskArtifact artifact = findOneProduct(taskId);
        if (Objects.isNull(artifact)){
            return null;
        }
        String authId = artifact.getAuthId();
        if (Objects.isNull(authId)){
            return artifact;
        }
        String artifactType = artifact.getArtifactType();
        AuthThird authServer = thirdServer.findOneAuthServer(authId);
        artifact.setAuth(authServer);

        if (artifactType.equals(PipelineFinal.TASK_UPLOAD_SSH)){
            AuthHost oneAuthHost = hostServer.findOneAuthHost(authId);
            artifact.setAuth(oneAuthHost);
        }
        return artifact;
    }

    /**
     * 删除流水线推送制品
     * @param ProductId 流水线推送制品id
     */
    @Override
    public void deleteProduct(String ProductId) {
        productDao.deleteProduct(ProductId);
    }

    /**
     * 更新推送制品信息
     * @param taskArtifact 信息
     */
    @Override
    public void updateProduct(TaskArtifact taskArtifact) {
        TaskArtifactDo productEntity = BeanMapper.map(taskArtifact, TaskArtifactDo.class);
        productDao.updateProduct(productEntity);
    }

    /**
     * 查询推送制品信息
     * @param artifactId id
     * @return 信息集合
     */
    @Override
    public TaskArtifact findOneProduct(String artifactId) {
        TaskArtifactDo oneProduct = productDao.findOneProduct(artifactId);
        return BeanMapper.map(oneProduct, TaskArtifact.class);
    }

    /**
     * 查询所有流水线推送制品
     * @return 流水线推送制品列表
     */
    @Override
    public List<TaskArtifact> findAllProduct() {
        List<TaskArtifactDo> allProduct = productDao.findAllProduct();
        return BeanMapper.mapList(allProduct, TaskArtifact.class);
    }

    @Override
    public List<TaskArtifact> findAllProductList(List<String> idList) {
        List<TaskArtifactDo> allProductList = productDao.findAllProductList(idList);
        return BeanMapper.mapList(allProductList, TaskArtifact.class);
    }

    private Object findAuth(String id){
        AuthThird oneAuthServer = thirdServer.findOneAuthServer(id);
        if (oneAuthServer != null){
            return oneAuthServer;
        }
        return hostServer.findOneAuthHost(id);
    }

}
