package cd.casic.module.terminal.service.impl;

import cd.casic.framework.security.core.LoginUser;
import cd.casic.framework.security.core.util.SecurityFrameworkUtils;
import cd.casic.module.terminal.controller.dto.TerminalAccessDTO;
import cd.casic.module.terminal.controller.dto.TerminalTransferDTO;
import cd.casic.module.terminal.dal.redis.TerminalRedisDAO;
import cd.casic.module.terminal.define.cache.TerminalCacheKeyDefine;
import cn.orionsec.kit.lang.id.UUIds;
import cn.orionsec.kit.lang.utils.Valid;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;


/**
 * 终端连接服务
 *
 */
@Slf4j
@Service
public class TerminalService{


    @Resource
    private TerminalRedisDAO terminalRedisDAO;

    public String getTerminalAccessToken() {
        LoginUser user = Valid.notNull(SecurityFrameworkUtils.getLoginUser());
        log.info("HostTerminalService.getTerminalAccessToken userId: {}", user.getId());
        String accessToken = UUIds.random19();
        TerminalAccessDTO access = TerminalAccessDTO.builder()
                .userId(user.getId())
                .username(SecurityFrameworkUtils.getLoginUserNickname())
                .build();
        // 设置 access 缓存
        String key = TerminalCacheKeyDefine.TERMINAL_ACCESS.format(accessToken);
        terminalRedisDAO.setJson(key, TerminalCacheKeyDefine.TERMINAL_ACCESS, access);
        return accessToken;
    }

    public String getTerminalTransferToken() {

        LoginUser user = Valid.notNull(SecurityFrameworkUtils.getLoginUser());
        log.info("HostTerminalService.getTerminalTransferToken userId: {}", user.getId());
        String transferToken = UUIds.random19();
        TerminalTransferDTO transfer = TerminalTransferDTO.builder()
                .userId(user.getId())
                .username(SecurityFrameworkUtils.getLoginUserNickname())
                .build();
        // 设置 transfer 缓存
        String key = TerminalCacheKeyDefine.TERMINAL_TRANSFER.format(transferToken);
        terminalRedisDAO.setJson(key, TerminalCacheKeyDefine.TERMINAL_TRANSFER, transfer);
        return transferToken;
    }

}
